/*
 * Project:        Exercise 7.1d
 * File:           exercise_7_1d.C
 * Author:         Benno List
 * Version (date): 1.0 (23.06.2013)
 *
 * Copyright (C) 2013, Benno List
 * All rights reserved.
 *
 * Description:
 * A ROOT macro for interactive ROOT sessions. See the READE file for
 * more details.
 *
 * This code is distributed with the solution manual to the book
 *
 * Data Analysis in High Energy Physics: A Practical Guide to Statistical Methods,
 * Wiley-VCH (2013),
 * O. Behnke, K. Kroeninger, G. Schott, Th. Schorrner-Sadenius (editors)
 */

int newtonit2 (int nit, double& e1, double& e2, double& l, 
               double e1m, double e2m, double s1, double s2, double r, double psi, double m) {
  for (int n = 0; n < nit; ++n) {
    double e1try = e1, e2try=e2, ltry=l;
    int notok = newtonit (1, e1try, e2try, ltry, e1m, e2m, s1, s2, r, psi, m);
    if (notok) return 1;
    // get the direction vector p = (p1, p2) for the full Newton step
    double p1 = e1try-e1, p2 = e2try-e2;

    double mu = fabs(ltry);
    double phi1start = phi1 (e1, e2, e1m, e2m, s1, s2, r, psi, m, mu);
    double Dphi1calc = Dphi1 (e1, e2, e1m, e2m, s1, s2, r, psi, m, mu, p1, p2);

    double alpha = 1;
    double c1 = 0.1;
    // Armijo condition Eq. (7.86)
    while ((alpha > 1E-6) && 
           (phi1 (e1+alpha*p1, e2+alpha*p2, e1m, e2m, s1, s2, r, psi, m, mu) > phi1start+c1*alpha*Dphi1calc)) {
      alpha *= sqrt(0.5);
    }
    e1 += alpha*p1;
    e2 += alpha*p2;
    l += alpha*(ltry-l);
  }
  return 0;
}

// Calculate the the merit function phi_1 = f + mu*|c| 
double phi1 (double e1, double e2, 
             double e1m, double e2m, double s1, double s2, double r, double psi, double m,
             double mu) {
  return (pow((e1-e1m)/s1, 2) - 2*r*(e1-e1m)*(e2-e2m)/(s1*s2) + pow((e2-e2m)/s2, 2))/(1-r*r)
         + mu*fabs (2*e1*e2*(1-cos(psi))-m*m);
}

// Calculate the directional derivative D(phi_1, p) for directions p that satisfy A*p = -c
double Dphi1 (double e1, double e2, 
              double e1m, double e2m, double s1, double s2, double r, double psi, double m,
              double mu, double p1, double p2) {
  return 2*(    ((e1-e1m)/(s1*s1) - r*(e2-e2m)/(s1*s2))*p1 + 
            (- r*(e1-e1m)/(s1*s2) +   (e2-e2m)/(s2*s2))*p2)/(1-r*r)
         - mu*fabs (2*e1*e2*(1-cos(psi))-m*m);
}

TH2F *plotimprovement2 (int nit, 
                        double nx=160, double xl=0, double xr=16, 
                        double ny=100, double yb=2, double yt=12) {
  stringstream title;
  title << "Log_{10} (Distance Ratio) for " << nit << " Iterations + Stepsize Control;E_{1};E_{2}";
  TH2F *h = new TH2F ("improvement2", title.str().c_str(), nx, xl, xr, ny, yb, yt);
  for (int i = 1; i <= h->GetNbinsX(); ++i) {
    for (int j = 1; j <= h->GetNbinsY(); ++j) {
      double e1 = h->GetXaxis()->GetBinCenter (i);
      double e2 = h->GetYaxis()->GetBinCenter (j);
      double d0 = sqrt(pow (e1-6.14827646488922230, 2)+pow (e2-4.06618019582670698, 2));
      double l = 0;
      newtonit2 (nit, e1, e2, l, 6, 7, 1, 1.5, -0.5, 1.570796326794897, sqrt(50.0));
      double d1 = sqrt(pow (e1-6.14828, 2)+pow (e2-4.06618, 2));
      h->SetBinContent (i, j, log10(d1/d0));
    }
  }
  return h;
}



